/* ***********************************************************************
 *
 *  udefs.hpp: declaration of the main Ubit classes 
 *  Ubit GUI Toolkit - Version 6
 *  (C) 2009 | Eric Lecolinet | TELECOM ParisTech | http://www.enst.fr/~elc/ubit
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************/

#ifndef _udefs_hpp_
#define	_udefs_hpp_ 1

#include <string>
#include <vector>
#include <list>

// <ubit/ubit_features.h> defines crucial compilation options. It is automatically 
// generated by 'configure' or 'umake' (which are in the Ubit base directory).
#include <ubit/ubit_features.h>

#define UBIT5 (UBIT_VERSION_MAJOR >= 5)
#define UBIT6 (UBIT_VERSION_MINOR >= 6)
#define null NULL

namespace ubit {

  typedef char UChar;

  class UObject;     // base class of all objects
  class UNode;       // base class of all objects that can be added to the scene graph
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // elements (groups, boxes and widgets)
  
  class UElem;    // base class of element nodes (derives from UNode)
  class UBox;
  class ULabel;
  class UButton;
  class UItem;
  class UTextfield;
  class UTextarea;
  class UFlowbox;
  class UCardbox;
  class UCombobox;
  class UListbox;
  class UAlertbox;
  class UFilebox;
  class UDocbox;
  class UTreebox;
  class UTreenode;
  class UPalette;
  class USlider;
  class UScrollbar;
  class UScrollpane;
  class UZoompane;
  class UTable;
  class UDoc;
  class UGlcanvas;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // windows and menus (derive from UBox)
  
  class UWin;
  class UFrame;
  class UDialog;
  class UMenubar;
  class UMenu;
  class UPopmenu;
  class UPiemenu;
  class UCtlmenu;
  class USubwin;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // 3D widgets (derive from UBox)
  
  class U3Dcanvas;
  class U3Dbox;
  class U3Dwin;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // data (strings, images, pixmaps, symbols)
  
  class UData;     // base class of data nodes (derives from UNode)
  class UStr;
  class UIma;
  class UPix;
  class USymbol;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // attributes
  
  class UAttr;  // base class of attributes nodes (derives from UNode)
  class UAttrList;
  class UCall;       // callback objects
  class UColor;
  class UAlpha;
  class UBackground;
  class UBorder;
  class UFont;
  class UFontFamily;
  class UFontMetrics;
  class UFontDesc;
  class UPos;
  class U3Dpos;
  class UScale;
  class USize;
  class UPadding;
  class UOrient;
  class UHalign;
  class UValign;
  class UHspacing;
  class UVspacing;
  class UCursor;
  class UChoice;
  class UEdit;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // application, display and global configuration
  
  class UAppli;
  class UDisp;
  class UConf;
  class UOption;
  class USelection;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // event conditions
  
  class UCond;
  class UOn;
  class UFlag;
  class UFlagdef;
  class UPropdef;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // events

  class UEvent;
  class UInputEvent;
  class UMouseEvent;
  class UKeyEvent;
  class UWheelEvent;
  class UViewEvent;
  class UPaintEvent;
  class UResizeEvent;
  class UWinEvent;
  class UUserEvent;
  class UMessageEvent;
  class UTimerEvent;  
  class USysWMEvent;

  class UEventFlow;
  
  class UKey;  // key codes
  class UMod;  // modifier masks
   
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // geometry
  
  class UUnit;
  class ULength;
  class UPoint;
  class UDimension;
  class UShape;
  class ULine;
  class URect;
  class UEllipse;
  class UArc;
  class UPolygon;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // graphics & rendering
  
  class UGraph;
  class URgba;
  class UUpdate;
  class URenderContext;
  class UGlcontext;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // errors
  
  class UError;
  class UErrorHandler;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // numbers
  
  class UInt;
  class UFloat;
  class UDouble;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // metaclasses
  
  class UClass;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // view and styles
  
  class UView;
  class UViewStyle;
  class UStyle;
  class UStyleSheet;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // arglists
  
  class UArgs;
  class UChild;
  class UParent;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // file and network
  
  class UFileInfo;
  class UFileDir;
  class USocket;
  class USource;
  class UTimer;
  class UMessage;
  class UMessagePort;
  class UMessagePortMap;
  class UMService;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // implementation
  
  class UUpdateContext;
  class UWinUpdateContext;
  class UBehavior;
  class UWinImpl;
  class USoftwinImpl;
  class UHardwinImpl;
  class UHardIma;
  class UHardPix;
  class UHardFont;
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  /* compatibility with Ubit5
  
   typedef UNode UBrick;
   typedef UAttr UProp;
   typedef UBackground UBgcolor;
   
   */
}

#endif
