/* ==================================================== ======== ======= *
 *
 *  uupixmaps.cpp
 *  a wrapping for using some of the great FVWM pixmaps in the Ubit toolkit
 *
 * ==================================================== ======== ======= *

Mini XPM icon package (for fvwm).
Version 1
Free (F) by Ludvig A. Norin 1995
emph@ludd.luth.se

This package contains 14x14 pixel icons
for use with the FVWM 2.x distribution.
The icons might of course be used in just
about any other purpose you can think of,
but since FVWM is such a great wm, I
really suggest that you use them in it,
rather than putting them in your private
.JPG/.GIF pictures just for the fun of it.

The icons are small enough to give the
FVWM menus a spiffy outlook. The icons
are optimized to use as few colors as
possible, and they will totally occypy
14 entrys of the colormap.
*/

#include <ubit/ubit_features.h>
#include <ubit/uxpm.hpp>
#include <ubit/upix.hpp>
#define NAMESPACE_UBIT namespace ubit {
NAMESPACE_UBIT

const char *UXpm::select[] = {
  " 16 16 3 1",
  ". c #000000",
  "# c None s None",
  "a c #ffffff",
  "################",
  "####.###########",
  "####..##########",
  "####.a.#########",
  "####.aa.########",
  "####.aaa.#######",
  "####.aaaa.######",
  "####.aaaaa.#####",
  "####.aaaaaa.####",
  "####.aaaa....###",
  "####.a..a.######",
  "####..#.aa.#####",
  "####.###.a.#####",
  "########.aa.####",
  "#########.a.####",
"################"};


const char * UXpm::bomb[] = {
"16 16 6 1",
"       c None s None",
".      c black",
"X      c #808080",
"o      c #c0c0c0",
"O      c yellow",
"#      c white",
"                ",
"       ...      ",
"      .   .     ",
"     .     .    ",
"    ...     .   ",
"   .....    .   ",
"  .X#o...    .  ",
" X.#X....X   .  ",
" ..oX.....   O O",
" .......O. O OO ",
" ......Xo.  OOO ",
" X.....X.X  O   ",
"  .......       ",
"   X...X        ",
"                ",
"                "};

const char * UXpm::clock[] = {
"16 14 4 1",
"       c None s None",
".      c Black",
"X      c White",
"o      c gray85",
"      .....     ",
"    ..XXXXX..   ",
"   .oXXX.XXXo.  ",
"   .XXXX.XXXX.  ",
"  .XXXXX.XXXXX. ",
"  .XXXXX.XXXXX. ",
"  .XXXX..XXXXX. ",
"  .XXXXX..XXXX. ",
"  .XXXXX.X.XXX. ",
"   .XXXXXXXXX.  ",
"   .oXXXXXXXo.  ",
"    ..XXXXX..   ",
"      .....     ",
"                "};

const char * UXpm::colors[] = {
"16 14 7 1",
"       c None s None",
".      c gray50",
"X      c red",
"o      c yellow",
"O      c blue",
"+      c black",
"#      c white",
"                ",
"                ",
" .............. ",
" .XXXXooooOOOO.+",
" .XXXXooooOOOO.+",
" .XXXXooooOOOO.+",
" .XXXXooooOOOO.+",
" .####....++++.+",
" .####....++++.+",
" .####....++++.+",
" .####....++++.+",
" ..............+",
"  ++++++++++++++",
"                "};

const char * UXpm::edit[] = {
"16 14 5 1",
"  c None s None",
". c black",
"# c gray50",
"i c white",
"o c red",
" #########      ",
" #iiiiiii#.     ",
" #i#####i#i.    ",
" #iiiiiii#...  o",
" #i#####iiii. oo",
" #iiiiiiiiii.oo.",
" #i########ioo. ",
" #iiiiiiiiioo.  ",
" #i#######.o.   ",
" #iiiiiii....   ",
" #i#####...i.   ",
" #iiiii.iiii.   ",
" #iiiiiiiiii.   ",
" ............   "
};


const char * UXpm::eyes[] = {
"16 14 5 1",
"       c None s None",
".      c black",
"O      c gray50",
"X      c gray85",
"o      c white",
"                ",
"                ",
"  ....   ....   ",
" .XooX. .XooX.  ",
".XooooX.XooooX. ",
".oooooo.oooooo.O",
".oooooo.oooooo.O",
".o..ooo.o..ooo.O",
".X..ooX.X..ooX.O",
" .XooX.O.XooX.OO",
"  ....OO ....OO ",
"   OOOO   OOOO  ",
"                ",
"                "};

const char * UXpm::mail[] = {
"16 14 4 1",
"       c None s None",
".      c black",
"X      c white",
"o      c gray50",
"                ",
"                ",
" .............  ",
" .XXXXXXXXXXX.o ",
" ..XXXXXXXXX..o ",
" .X.XXXXXXX.X.o ",
" .XX.XXXXX.XX.o ",
" .XX..XXX..XX.o ",
" .X.XX...XX.X.o ",
" ..XXXXXXXXX..o ",
" .............o ",
"  ooooooooooooo ",
"                ",
"                "};

const char * UXpm::book[] = {
"16 14 6 1",
"       c None s None",
".      c Black",
"X      c Red",
"o      c Yellow",
"O      c gray50",
"+      c White",
"       ..       ",
"      .XX.      ",
"     .XXXX.     ",
"    .XXXoXX.    ",
"   .XXXoXoXX.   ",
"  .XXXXXoXoX.   ",
" .XXXXXXXoX.O   ",
" ..XXXXXXX.+.O  ",
" .+.XXXXX.+.O   ",
"  .+.XXX.+.O    ",
"   .+.X.+.O     ",
"    .+.+.O      ",
"     .+.O       ",
"      .O        "};

const char *UXpm::paint[] = {
"    16    14       6            1",
"  c None s None",
". c black",
"# c gray50",
"a c red",
"i c white",
"o c blue",
"#########       ",
"#iiiiiii#.      ",
"#iiiiiii#i.    o",
"#iiiiiii#...  oo",
"#iiiiiiiiii. oo.",
"#iiaiiiiiii.oo. ",
"#iiiaaiiiiioo.  ",
"#iiiiaaiiioo.   ",
"#iiiaaaiioo.    ",
"#iiaaaii#...    ",
"#iiaaii#i.i.    ",
"#iaaaaaa.ii.    ",
"#iaaaaaiiii.    ",
"............    "
};

const char* UXpm::palette[] = {
"16 16 9 1 0 0",
"       s none  m none  c none",
".      s iconColor1    m black c black",
"X      s iconColor6    m white c yellow",
"o      s iconColor8    m black c magenta",
"O      c #0000FFFF0000",
"+      s iconColor4    m white c green",
"@      c #808080808080",
"#      s iconColor3    m black c red",
"$      s iconColor5    m black c blue",
"    ...         ",
"   .XXX..       ",
"  .XXXXXX.      ",
" .XXXoooXX.    .",
".XXXXooo.X..  ..",
".XXO+@..XXX. .. ",
".X+++@XXXXXX..  ",
".X@@@XXXX.@..X. ",
".XXXXXXXX....XX.",
" .X###.XXX..XXX.",
" .X###.XXXXXXXX.",
"  .X..X$$$$$XXX.",
"   .XXX$$$$XXX. ",
"    ...XXXXX..  ",
"   @.. .....    ",
" @@..           "};

const char* UXpm::term[] = {
"    16    14        6            1",
"  c None s None",
". c Black",
"# c gray50",
"a c blue",
"b c White",
"c c gray85",
"     #########  ",
"    #cccccccc#. ",
"   #bbbbbbbb##. ",
"   #b......b##. ",
"   #b.aaaaab##. ",
"   #b.aaaaab##. ",
"   #b.aaaaab##. ",
"   #bbbbbbbb#.##",
"   #.........#c.",
"  #bbbbbbbbb#c#.",
"  ##.#.#.#.#.##.",
" #c.c.c.c.c.c.. ",
"#bbbbbbbbbbbc.  ",
".............   "
};

const char * UXpm::bricks[] = {
"16 14 5 1",
"       c None s None",
".      c gray50",
"X      c yellow",
"o      c black",
"O      c red",
"                ",
"                ",
"     ......     ",
"     .XXXXo     ",
"     .XOOOo     ",
"     .XOOOo     ",
"     .XOOOo     ",
"......OOOOoooooo",
".XXXXOXXXXOXXXXo",
".XOOOOXOOOOXOOOo",
".XOOOOXOOOOXOOOo",
".XOOOOXOOOOXOOOo",
".ooooooooooooooo",
"                "
};

const char * UXpm::camera[] = {
"16 14 4 1",
"       c None s None",
".      c black",
"X      c white",
"o      c gray50",
"      ......    ",
"     .XXXXXX.   ",
" ....Xo...oX... ",
".XXXX.......XXX.",
".......ooo......",
".Xoo..oXXoo..oo.",
".Xoo..oXooo..oo.",
".Xoo..ooooo..oo.",
".Xooo..ooo..ooo.",
".Xooo.......ooo.",
".Xooooo...ooooo.",
" .............. ",
"                ",
"                "
};

const char * UXpm::windows[] = {
"16 14 4 1",
"       c None s None",
".      c Black",
"X      c blue",
"o      c white",
"..........      ",
".XXXXXXXX.      ",
".oooooooo.      ",
".ooooo..........",
".ooooo.XXXXXXXX.",
".ooooo.oooooooo.",
".oo..........oo.",
".oo.XXXXXXXX.oo.",
"....oooooooo.oo.",
"   .oooooooo.oo.",
"   .oooooooo.oo.",
"   .oooooooo....",
"   .oooooooo.   ",
"   ..........   "
};

  /* ==================================================== ======== ======= */
  
  const char *upix_rball_xpm[] = {
    "10 10 4 1",
    "       c None s None",
    ".      c red",
    "X      c white",
    "o      c gray50",
    "          ",
    "          ",
    "   ...    ",
    "  .XX..   ",
    "  .XX..o  ",
    "  .....o  ",
    "   ...oo  ",
    "    ooo   ",
    "          ",
  "          "};
  
  const char *upix_ellipsis_xpm[] = {
    " 13 13 2 1",
    ". c #ffffff",
    "# c #000000",
    ".............",
    ".............",
    ".............",
    ".............",
    ".............",
    ".............",
    "...#.#.#.#...",
    ".............",
    ".............",
    ".............",
    ".............",
    ".............",
  "............."};
  
  /* ==================================================== ======== ======= */
  
  const char *upix_cross_xpm[] = {
    " 16 16 23 1",
    ". c None s None",
    "# c #ef4a52",
    "a c #ef5252",
    "b c #bd0000",
    "c c #ff0000",
    "d c #d6525a",
    "e c #733939",
    "f c #ce0000",
    "g c #ce7373",
    "h c #c60000",
    "i c #732121",
    "j c #e70000",
    "k c #c68c8c",
    "l c #730808",
    "m c #f72129",
    "n c #8c0000",
    "o c #e73939",
    "p c #9c0000",
    "q c #947373",
    "r c #ad0000",
    "s c #845a5a",
    "t c #7b4242",
    "u c #733131",
    "................",
    "................",
    "................",
    "....##.....a#...",
    "....bcd...dcb...",
    "....efcg.gche...",
    ".....ijckcji....",
    "......lcccl.....",
    ".......ccc......",
    "......mcncm.....",
    ".....ocpqpco....",
    "....acrs.srca...",
    "....fbt...tbf...",
    "....uu.....uu...",
    "................",
  "................"};
  
  /* ==================================================== ======== ======= */
  
  const char *upix_check_xpm[] = {
    " 16 16 32 1",
    ". c None s None",
    "# c #63ef73",
    "a c #6bff7b",
    "b c #08ff31",
    "c c #009c08",
    "d c #08b529",
    "e c #31424a",
    "f c #10d631",
    "g c #183931",
    "h c #c6bdc6",
    "i c #10f731",
    "j c #18ff39",
    "k c #18f739",
    "l c #083921",
    "m c #bdadbd",
    "n c #004210",
    "o c #21ff42",
    "p c #29ff42",
    "q c #a5a5ad",
    "r c #8c9494",
    "s c #005208",
    "t c #39ef52",
    "u c #006321",
    "v c #737b7b",
    "w c #006b10",
    "x c #002100",
    "y c #5a636b",
    "z c #008c18",
    "A c #5a7b6b",
    "B c #424a52",
    "C c #00a521",
    "D c #294242",
    "................",
    "................",
    "................",
    "............#a..",
    "............bc..",
    "...........bde..",
    "..........bfgh..",
    "...ij....jklm...",
    "...nop..ponq....",
    "...rsottour.....",
    "....vwojoxm.....",
    ".....yzozA......",
    "......BCB.......",
    ".......D........",
    "................",
  "................"};
  
const char* UXpm::leftarrow[] = {
  "12 11 2 1",
  ". c None s None",
  "# c orange",
  "............",
  "....#.......",
  "...##.......",
  "..#########.",
  ".##########.",
  "###########.",
  ".##########.",
  "..#########.",
  "...##.......",
  "....#.......",
  "............",
};

const char *UXpm::rightarrow[] = {
  "12 11 2 1",
  ". c None s None",
  "# c orange",
  "............",
  ".......#....",
  ".......##...",
  ".#########..",
  ".##########.",
  ".###########",
  ".##########.",
  ".#########..",
  ".......##...",
  ".......#....",
  "............",
};

const char * UXpm::uparrow[] = {
  "11 12 2 1",
  "       c None s None",
  ".      c orange",
  "     .     ",
  "    ...    ",
  "   .....   ",
  "  .......  ",
  " ......... ",
  "   .....   ",
  "   .....   ",
  "   .....   ",
  "   .....   ",
  "   .....   ",
  "   .....   ",
  "           "
};

const char *UXpm::downarrow[] = {
   "11 12 2 1",
   "       c None s None",
   ".      c orange",
   "           ",
   "   .....   ",
   "   .....   ",
   "   .....   ",
   "   .....   ",
   "   .....   ",
   "   .....   ",
   " ......... ",
   "  .......  ",
   "   .....   ",
   "    ...    ",
   "     .     "
};

const char *UXpm::bigFolder[] = {
    "32 32 41 2",
    "AA c #FFFFFFFFFFFF",
    "BA c #EFEFEFEFEFEF",
    "CA c #DEDEF7F7FFFF",
    "DA c #DEDEEFEFFFFF",
    "EA c #DEDEDEDEDEDE",
    "FA c #CECEEFEFFFFF",
    "GA c #CECECECECECE",
    "HA c #BDBDEFEFFFFF",
    "IA c #BDBDDEDEFFFF",
    "JA c #BDBDD6D6EFEF",
    "KA c #B5B5CECEE7E7",
    "LA c #ADADE7E7FFFF",
    "MA c #ADADDEDEFFFF",
    "NA c #ADADD6D6EFEF",
    "OA c #ADADADADADAD",
    "PA c #A5A5CECEEFEF",
    "AB c #9C9CDEDEFFFF",
    "BB c #9C9CCECEEFEF",
    "CB c #9C9CBDBDE7E7",
    "DB c #9C9C94949494",
    "EB c #9494CECEF7F7",
    "FB c #8C8CD6D6FFFF",
    "GB c #8C8CCECEFFFF",
    "HB c #8C8CC6C6EFEF",
    "IB c #848484848484",
    "JB c #7B7BCECEFFFF",
    "KB c #7373ADADDEDE",
    "LB c #737384849C9C",
    "MB c #6B6BC6C6FFFF",
    "NB c #6363BDBDFFFF",
    "OB c #5A5A8484ADAD",
    "PB c #42427B7BB5B5",
    "AC c #39398484C6C6",
    "BC c #39397B7BC6C6",
    "CC c #31317373BDBD",
    "DC c #29297373BDBD",
    "EC c #21216B6BBDBD",
    "FC c #18186B6BBDBD",
    "GC c #08086B6BBDBD",
    "HC c #08086363ADAD",
    "IC c None",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
    "ICICICICICICICICICICICICICICICICIBIBIBIBIBIBIBIBIBIBICICICICICIC",
    "ICICICICICICICICICICICICICICICIBGABAAAAAAAAAAAAAEAIBIBICICICICIC",
    "ICICICICICICICICICICICICICICIBIBGABABAEABABAEABAAAEAIBIBICICICIC",
    "ICIBIBIBIBIBIBIBIBIBIBIBIBIBOAEAAABAEABAEAEABAEAEAAAGADBIBIBIBIC",
    "ICIBBAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAEAIBIC",
    "ICLBACBCBCBCBCBCBCBCBCBCJAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAIBIC",
    "ICPBCBFAFAFAFAFAFAFAFABBCCKAEABAEAEABAEABAEABAEABAEABAEABABAIBIC",
    "ICCCFAPABBBBPABBPAPANAFACBCCEAEABAEAEABAEABAEABAEABAEABABABAIBIC",
    "ICDCJAMABBMAMAMANAMAMANADABBACBCDCBCBCDCBCDCBCDCBCDCBCDCBCCBIBIC",
    "ICDCFAMAABPAMAMALAIAMAHAMADADACADAFACADAFADAFAFAFAFAFAFAJAKBOBIC",
    "ICDCJABBBBABNANANANANANANANAMANAMANAABNAABNANAEBBBBBBBHBIANADCIC",
    "ICCCJAMABBBBABNAMANALANALANANAMANAMANANANAABBBMABBGBBBHBABJADCIC",
    "ICDCJAMAABMAMAMAHALAHAHAIAHAHAMAHAMAHALAMAMAMAMAABNAABABGBFAECIC",
    "ICDCMANAABMANALAMAHAIAHAHAIAHAHANAHAMAHAMAHAMANAMAABEBABBBIAECIC",
    "ICDCNAMAHBNANAMANANALANANALANANAHANAJAMANANANANAABNAEBBBFBJADCIC",
    "ICDCNAABNAFBNANALANANAHANAHANAHANALALANALANAABNANAFBHBGBBBLAFCIC",
    "ICDCPAMAFBABFBLAMALAHALAHANAHAIALAHANAHALALALAABABABFBNAFBIAECIC",
    "ICDCNAABGBABMAABMAMAMAHAMAHAMALAHAMAHAMAMALAMAMAABABABJBEBIAFCIC",
    "ICDCBBGBHBHBHBBBFBNANAABNAABNANAABNANAABNANANAABHBHBHBHBGBNAECIC",
    "ICDCBBEBJBHBJBHBFBFBFBNAFBNAFBNANAFBABNAFBFBHBFBFBHBJBHBMBJAFCIC",
    "ICFCBBFBHBJBGBFBFBFBNAFBABABABFBFBABABFBABABFBABHBJBGBMBJBLAFCIC",
    "ICDCHBGBMBJBJBHBJBGBFBGBFBFBABFBABGBHBFBGBHBFBJBGBJBHBNBJBNAFCIC",
    "ICFCBBGBNBNBNBMBHBJBJBJBHBGBJBHBJBJBFBGBJBFBMBHBJBMBNBNBNBMAFCIC",
    "ICDCKBHBPAPAPABBNABBPANANAPAPANAPAPAPANAPAPANABBBBPABBPAPAKBFCIC",
    "ICGCGCHCGCHCHCGCHCGCHCGCHCGCHCGCHCHCGCHCGCHCGCHCGCHCGCHCHCGCHCIC",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
    "ICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICICIC",
  };
  
const char* UXpm::bigDoc[] = {
  "25 36 10 1",
  "A c #FFFFFFFFFFFF",
  "B c #EFEFEFEFEFEF",
  "C c #DEDEDEDEDEDE",
  "D c #CECECECECECE",
  "E c #BDBDBDBDBDBD",
  "F c #ADADADADADAD",
  "G c #9C9C9C9C9C9C",
  "H c #848484848484",
  "I c #6B6B6B6B6B6B",
  "J c None s None",
  "BBCBCBCBCCBCCCCCBJJJJJJJJ",
  "BCBCBCBCCCCCDDEEEBJJJJJJJ",
  "CBCCCCCCCCDDCDEEFEBJJJJJJ",
  "BCCBCCCCCDCCDDDEFFDBJJJJJ",
  "CBCCCCCCCCCDCDDDFGCDBJJJJ",
  "CCBCBCBCCCCCCCDDEHCBDBJJJ",
  "BCCBCBCCBCCCCCCDEGCACDBJJ",
  "CCBCBCCBCCCCCCCDEHBJJBDBJ",
  "BBCBCBCBCBCCCCCDDGFFFEDDB",
  "BCBCBCBCBCCBCCCCDFHIIIIGF",
  "BBCBBBCBCBBCBCCCDEGGGGGHF",
  "BCBBBCBBBBCBCBCCCDEEEEEEF",
  "BBBCBBCBCBBCBCBCCCCDDDDDD",
  "BBBBCBBBBCBBBBCBBCCCCCCCD",
  "BBBBBBBBBBBBBBBCBBCBCCCDC",
  "BBBBBBBBBBCBBCBBCBCBCCCCC",
  "BBBBBBBBBBBBBBBBBBBCBCBCC",
  "ABBABBBBBBBBBBBBBBBBBBCCB",
  "BABBABBABBABBBBBBBBBCBBCB",
  "ABBABBABBABBABBBBBBBBBCBB",
  "AABABABABBBABBABBBBBBBBBC",
  "BABABABBABABBABBABABBBBBB",
  "ABABABABBABBABBABBBABBBBB",
  "BAABABABAABABBABBABBBABBB",
  "ABAABABABBABABBABBABBBABA",
  "AAABABABAABABAABBABBABBAB",
  "ABAABAAABAABABBAABABBABBA",
  "BAABAAABABABABAABABBABBAB",
  "AABAABAABABABABABABABABBA",
  "AAAABAABAAABAAABABABABABA",
  "AAAAAABAABAABABAABAABABAB",
  "AABABAAABAAAABAABAABABABA",
  "AAAAAAAAAABAAABAABAABAAAA",
  "AAAAAABAAAAABAAAAABAABABA",
  "AAAAAAAABAAAAABAAAABAABAA",
  "AAABAAAAAAAAAAAAABAABAAAA",
};


const char *UXpm::bigText[] = {
  "28 38 31 2",
  "AA c #FFFFFFFFFFFF",
  "BA c #EFEFEFEFEFEF",
  "CA c #E7E7DEDEE7E7",
  "DA c #DEDEDEDEDEDE",
  "EA c #CECED6D6DEDE",
  "FA c #CECECECECECE",
  "GA c #C6C6CECED6D6",
  "HA c #BDBDC6C6D6D6",
  "IA c #BDBDC6C6CECE",
  "JA c #BDBDBDBDCECE",
  "KA c #BDBDBDBDBDBD",
  "LA c #ADADB5B5BDBD",
  "MA c #ADADADADADAD",
  "NA c #A5A5ADADBDBD",
  "OA c #9C9C9C9C9C9C",
  "PA c #94949C9CADAD",
  "AB c #8C8C9C9CB5B5",
  "BB c #8C8C9C9CADAD",
  "CB c #8C8C8C8C8C8C",
  "DB c #84848C8CA5A5",
  "EB c #7B7B9494B5B5",
  "FB c #7B7B8C8CB5B5",
  "GB c #7B7B8C8CADAD",
  "HB c #7B7B7B7B7B7B",
  "IB c #6B6B8484B5B5",
  "JB c #6B6B8484ADAD",
  "KB c #6B6B6B6B6B6B",
  "LB c #5A5A7B7BA5A5",
  "MB c #4A4A6B6BA5A5",
  "NB c #424263639C9C",
  "OB c None s None",
  "IBEBEBFBEBFBFBFBFBGBFBJBFBJBGBJBJBLBMBOBOBOBOBOBOBOBOBOB",
  "IABABABABADABABADABADADADADADAFAKAKABBMBOBOBOBOBOBOBOBOB",
  "JABADADADABAEADADAEADADAEAFAFAFAFAMAMAPANBOBOBOBOBOBOBOB",
  "LABADABADADADADADADADAEADADADAFAFAKAMAMALAMBOBOBOBOBOBOB",
  "JADABADABADABADADADADADADAFADADAFAFAMAOADANANBOBOBOBOBOB",
  "JABADAFAEADADADADADADAFAFADAFAFAFAFAMACBDABALANBOBOBOBOB",
  "JABADAMAMAMAIAMAMAKAMAKAMAOAMAMAKACBMAHBDAAABANANBOBOBOB",
  "LABADAFADADADADADADADADAFADAFAFAFAKAKAOADAAAAABALAMBOBOB",
  "IABABAKAMAKAKAMAMAFAMAMAKAKAKAMAKAOAOACBOAMAMAKAFAMAMBOB",
  "JABADADADADADADADADADADADADAFADAFADAKAMAHBKBKBKBHBCBOAMB",
  "JABABAMAKAKAKAKAMAMAMAMAFAMAMAOAMAOAKACBCBHBHBKBCBPAPACB",
  "IABABAEABADADADADADADABADABADADADADAFAFAKAKAKAKAKAKAMANA",
  "IABADAMAKAKAMAOAKAMADABABACABABACABADADAEAFADAFAFAFAKAKA",
  "JAAABABADABADABADADABADABABABADABADABADADADADADADAFAEAKA",
  "IABABABABABABABABABABABABABABABABABADABABADADADADADADAFA",
  "JAAABADABADABADADABADADABADADADADADABADADADADADADADADAFA",
  "IAAADAKAKAKALAKAKAMAFAMAKAEAFAKAMAKAKAIAKAMAKAMADADADAFA",
  "GABAAADABABACABABADABABACABADABABADABACABADADADADADACADA",
  "HAAABAKAMAMAKAFAKAKAKAKAKAKAKALAMAKAMAMAFAKAKAMAFAFADADA",
  "FAAABABAFABADABABADADABABADADADABADABABABABADADADADABADA",
  "HAAAAAMALAKAKAMAFAKAKAFALAMAKAKAFAMAMAMAKAKAMAFAKABABADA",
  "GAAABABABABAAADABABABABABABABABABACADABADABABADABABABACA",
  "GAAAAAKAMAKADAKAKAMAKAKAKADAKAKAMAKAFAKAKAKAMAKAMADABADA",
  "HAAABABABACABABADABABABABABABADABABABABABABADADABADABABA",
  "GAAAAAKADAFAKAKAKADAKAKAKAKAKAFAFAKAFAMAKAKAKAKAMAIAAADA",
  "FAAABADADABADADABABADADABABADABADABABABABABABABABABABABA",
  "HAAABAKAKAFAKAKAFAKAKAFAMAFAKAKAKAFAAABABAAABABAAABABABA",
  "IAAAAAAAAAAAAABAAABABAAABABAAAAABAAABAAABAAABAAABABAAABA",
  "GAAAAABAAABAAAAAAAAAAABAAAAABAAAAABAAABAAABAAABAAABABABA",
  "HAAABABABAAABABABAAABABABABAAABABAAABABABABABABAAABAAABA",
  "GAAAAAKAKAKADAMAFAFAKAFAKAKAKAKAKAFAKAMAMAKAKAFAAAAABABA",
  "IAAAAABABABABABABABABABAAABABABAAABABABABABABABABABAAABA",
  "GAAABAKAFAMAKAFAKAKAFAKAKAKADAKAKAKAFAFAFAKAKAKAKABABABA",
  "HAAAAABABABABABABABABAAABABABABABABABADAAABABAAABABAAABA",
  "GAAABAFAFAKADAFAKAMADAFAKAKAFAFAMAFAKAKAFABAAABAAAAAAABA",
  "GAAAAABAAABAAABABAAABAAABAAABAAADABAAAAABABAAAAAAABAAABA",
  "GAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABA",
  "GBPABBBBOABBBBOABBOABBOAOABBOAOABBOAOAGBOABBOADBOABBBBAB",
};


const char *UXpm::bigIma[] = {
  "39 31 30 2",
  /* colors */
  "AA c #FFFFFFFF2929",
  "BA c #CECECECEE7E7",
  "CA c #ADADB5B5E7E7",
  "DA c #9C9CB5B5CECE",
  "EA c #9494ADADEFEF",
  "FA c #8C8CADADCECE",
  "GA c #8C8CA5A5B5B5",
  "HA c #8C8C9C9CADAD",
  "IA c #8C8C9C9C9C9C",
  "JA c #84849C9CE7E7",
  "KA c #84847B7B8484",
  "LA c #7B7B9C9CCECE",
  "MA c #7B7B8C8CA5A5",
  "NA c #73737B7BBDBD",
  "OA c #73736B6B6B6B",
  "PA c #6B6B8C8CDEDE",
  "AB c #6B6B6B6B7373",
  "BB c #5A5A73739C9C",
  "CB c #5A5A6B6B7B7B",
  "DB c #4A4A4A4A3131",
  "EB c #424252526B6B",
  "FB c #42424A4A5252",
  "GB c #313142422929",
  "HB c #313131314A4A",
  "IB c #313129293939",
  "JB c #212121213939",
  "KB c #212121211010",
  "LB c #212118182929",
  "MB c #101018180808",
  "NB c #000000000000",
  /* pixels */
  "CABACACACACACACAEAEADACABACAJAJAPAPAPAPAPAPAPAPAPAPAPANAPAKBLBKBAAHBAAOAJBAALB",
  "CAGACACACADACACABACACACABABABACACAJAJAPAPAJANAPAJAPAPAJAPADBMBAAIBLAJAHBMAPAEB",
  "CADAEAFAEAEADACAFAEACACACACADABACACAPAJAPALAJAPAPAJALALBOAJBKBIBMBNAJBNBPANAPA",
  "EAEADAJAFAJAFAPAJAJAJAFACACACALAJAPAPAJAJAPAJAJAPAPAJAFBDBDBDBIBEBJBAAHBAAJAMB",
  "EADAPAPAJAJAJAFALAJAJAJAFAJAJAJAJAJAJAJAJAJAJAJAJAJAJAAANBIBDBMBIBHBMBGBMAHBJB",
  "FAJAPACAJAJAJAJAJAJAEAJAJAJAJAEAEAJAJAJAJAJAJAJAJAJAJAGBJBCAJBKBIBDBJBOAJALBHB",
  "EAFAJALAEADAJAFAEAEACAEACAEACAFALACAEAJAJAJAEACAJAJAJAMBEAJBKBJBKBKBHBJBEBJBHB",
  "JAGAEAEADAJAEAEAFACAJAEACAEACAJACAJACAEACAJACAJAJACAEAMBHAGAOAKBIBBBFBABJADAKB",
  "CAJADAJAEAEAEAEAJAEACAEACACAEACAEACACACAEACAEAJACACACALBCAJBGBJBGBJBIBAAKAFBFA",
  "JAFAJAEAFAEAEAEAEACAEACAEACACAEACACACAEACAJAEACACAEACACAHBEBDBAAOAFBKBGBPAMAGB",
  "CACAEAEACACACACACAEACACAEACACAEACAEACACACACACACACACACAEAOAABCAKAOAEBGALBIBCAMA",
  "EAJADAEACAJAEACACABAEAHBFBJBHBJBCACACACAHBHBFBFBFBHBEBEBNBFBABAAJBDBJBJBLBFBNB",
  "EACAGAGAHBIBIBJBBAFBJBJBJBHBJBDBHBFBHBDBJBJBHBJBJBHBGBHBKBEBDBJBOAJBGBJBKBJBHB",
  "CAGBDBHBJBDBDBDBHBDBHBDBJBHBGBEBDBHBDBEBJBDBEBDBHBGBOAEBJBDBNBHBFBDBMBJBMBOAKB",
  "GBLBIBGBJBJBJBJBGBHBDBHBEBGBCBGBFBJBHBGBHBHBGBHBDBEBGBDBDBHBLBOAJBHBDBIBHBKBKB",
  "GBHBJBDBDBGBCBGBHBDBEBKBOAEBDBHBJBGBHBDBHBDBKBEBJBOAFBFBHBIBIAGBDBKAHBDBGBFBDB",
  "BBOABBBBBBBBCBBBBBNACBMACBBBBBBBHABBBBCBBBBBLAMAGAMANAKADBKAMAJBHAOANBHBGBKBAA",
  "MALALALAMALAGANAGAGALAMALAGALANAGAHAPAGAGAGAGAMALAHALAFAMALAABIAOAEBGAMBOAMBDB",
  "HAMAMAHAGAGAHAGAMAGAGALAGAGADAGADAGAGACAPALADAGAGALAHAGAEAHAGAOAOAKBFAKAOAKBKB",
  "GAGAHAHAGAGALADAGADALAGAGADAGAGABBCAGADAGAGAGAGAPAGAGAGADAKAJBGBJBHBKAMAKBKBHB",
  "DAGAPADAPAGAGAGACAGADAPAGAGADAGAGADADAGAPAGADAGAGAGACAGAGADAEBKBDBDAGAHAMBOAMB",
  "GADAGAGAGAGAGAPAGAPAGAGAGADAGAGACAGAPAPAGADAGAGACACADACAGADADADADAIAGAGAOACBKB",
  "GAGADAGADAGAPAGAGAGAPAGAPAGAGADADAGAGAGADAGAGADAGAGAGAGACAGADAGADAGADAGAGAKBKB",
  "GADAGAGAGACAGAGADAGAGADAGAGAGAPAGADAGAGADAGADAGAGAGAPADAGAGAGAGAGAGABBIAIAGALB",
  "GALAGADAGAGAGAGADAGAGADAGAGAGAGADAGAGADADAPAGADAGAGADAGAGADAGABBIAIAHABBKAIAIA",
  "GABBGAGAGAGAGAGAPAGADAGAGAGAGAGAGAGADAGADAGAGAGAGADADAIAGAGAIAGAGABBHAKAHABBGA",
  "IAGAGABBGAGABBIAGAGAGAGAGAGAGAGAGAGAGAGAGAGAGADADAIADADAIAGAGAGAIAGAGAGAGAKAHA",
  "CBGABBGAIAGAGAIAIAGAGAGAGAGAIAGAGAGAGAIAIADADAGAGADADADAIAIAIAIAGAGAHAIAGAKAKA",
  "ABGAIAGAGAGAIAGAGAIADAIADAIAGAGAIAGAGAGADAIADADAIAIADADAIAIAIAIAIAIAGAIAIAIAKA",
  "IAIAIAIAIAIAGAIAIAIAIAIAIAIAGADAIAIADAOADADADADAIADADAIAIAIAIAIADAIAKAKAIADBCB",
  "HAABIAEBIAIAIAIAIAGAGAGAIAGADAIAGAIAOAIAIAIAOADADAIAOADADADADAIAKAOADAOAKAOAOA"};


const char *UXpm::bigDown[] = {
  "32 32 42 2",
"AA c #FFFFFFFFFFFF",
"BA c #EFEFEFEFF7F7",
"CA c #EFEFEFEFEFEF",
"DA c #DEDEDEDEEFEF",
"EA c #D6D6D6D6DEDE",
"FA c #CECECECEE7E7",
"GA c #BDBDBDBDDEDE",
"HA c #B5B5B5B5BDBD",
"IA c #ADADADADD6D6",
"JA c #ADADADADBDBD",
"KA c #A5A5A5A5CECE",
"LA c #94949494ADAD",
"MA c #8C8C8C8CBDBD",
"NA c #8C8C8C8CADAD",
"OA c #7B7B7B7BBDBD",
"PA c #7B7B7B7B9C9C",
"AB c #6B6B6B6BB5B5",
"BB c #6B6B6B6B9C9C",
"CB c #63636363ADAD",
"DB c #5A5A5A5AEFEF",
"EB c #5A5A5A5A9494",
"FB c #4A4A4A4ADEDE",
"GB c #4A4A4A4A9C9C",
"HB c #4A4A4A4A7B7B",
"IB c #42424242B5B5",
"JB c #424242428C8C",
"KB c #39393939CECE",
"LB c #31313131BDBD",
"MB c #313131319C9C",
"NB c #313131318484",
"OB c #29292929ADAD",
"PB c #212121218C8C",
"AC c #212121217B7B",
"BC c #18181818ADAD",
"CC c #181818189494",
"DC c #101010105A5A",
"EC c #080808089C9C",
"FC c #080808088484",
"GC c #080808086B6B",
"HC c #000000003939",
"IC c #000000002121",
"JC s bg c None",
"JCJCJCJCJCJCJCJCJCJCJCJCJCICHCHCHCHCICJCJCJCJCJCJCJCJCJCJCJCJCJC",
"JCJCJCJCJCJCJCJCJCICHCFCFCFCFCFCFCFCFCFCGCGCJCJCJCJCJCJCJCJCJCJC",
"JCJCJCJCJCJCJCJCGCFCFCFCFCFCFCFCFCFCFCFCFCFCFCGCJCJCJCJCJCJCJCJC",
"JCJCJCJCJCJCJCGCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCJCJCJCJCJCJCJC",
"JCJCJCJCJCJCGCFCFCFCFCFCGCGCGCGCGCGCFCFCFCFCFCFCFCFCJCJCJCJCJCJC",
"JCJCJCJCICFCFCFCFCFCFCGCGCBBNANANANAJBFCFCFCFCFCFCFCFCJCJCJCJCJC",
"JCJCJCHCFCFCFCFCFCFCFCGCNBAAAAAAAAAAEAFCFCFCFCFCFCFCFCFCJCJCJCJC",
"JCJCJCCCPBCCACFCFCFCFCGCNBCAAAAAAAAACAFCFCFCFCFCFCFCFCFCFCJCJCJC",
"JCJCGCNBNBNBPBCCACFCFCGCNBAAAAAAAAAAEAFCFCFCFCFCFCFCFCFCFCFCJCJC",
"JCICMBGBGBMBPBPBCCACFCGCNBCAAAAAAAAACAGCFCFCFCFCFCFCFCECFCECICJC",
"JCDCEBEBGBGBMBNBPBPBFCGCNBAAAAAAAAAAEAFCFCFCFCFCFCECECECECECFCJC",
"JCPBCBEBEBGBGBMBMBPBPBGCNBAAAAAAAACACAFCFCFCECFCFCCCCCECBCCCCCJC",
"JCGBCBCBCBEBEBGBMBMBPBDCNBCAAAAAAAAAEAFCFCFCCCECBCBCBCOBBCOBBCIC",
"ICCBOAOACBCBEBEBGBGBOBACHBCAAAAAAAAACAFCCCBCCCOBOBOBLBLBLBOBKBDC",
"HCMAMAOAOAOACBEBEBGBGBACHBAAAAAAAAAAEACCCCOBOBLBFBFBIBFBFBFBFBAC",
"HCKALAMAMAMAOACBCBEBEBNBHBAAAAAAAAAACACCOBOBLBFBFBDBDBDBDBDBFBNB",
"HCKAKAKAMAMAMAOAABCBEBHBEBCAAAAAAAAAEAOBOBOBLBIBDBDBDBDBDBDBDBNB",
"HCKAGAKAKAKAOABBEBEBEBHBHBAAAAAACAAACAFCDCACACMBLBIBFBFBDBIBFBAC",
"HCKAGAGAKAKAPAHAEAHALAPABBAAAAAAAAAAEABBBBJAFACAKALBLBKBLBFBKBDC",
"ICMAFAGAGAKAMAEAAAAAAAAACAAAAAAAAAAAAAAAAAAACAAAIABCOBOBOBOBOBHC",
"JCCBDAFAFAGALAJABAAAAAAAAAAAAAAAAAAACAAAAAAAAAFAOBCCBCBCOBBCOBJC",
"JCACDADAFAFAGAKALACAAAAAAAAABAAAAAAAAAAAAAAAGAECFCECECCCECBCFCJC",
"JCICGACADADAFAGAKALACAAAAAAAAAAAAAAAAAAAAALAFCFCFCFCECFCECECHCJC",
"JCJCJBBABADADADAGAGABBGAAAAAAAAAAABAAADAEBFCFCFCFCFCFCECFCFCJCJC",
"JCJCICMAAAAACADADAKAFCFCNACAAAAAAAAADAPBFCFCFCFCFCFCFCFCFCICJCJC",
"JCJCJCHCGBGACADAMAFCFCGCGCNBCAAAAAGAACFCFCFCFCFCFCFCFCFCICJCJCJC",
"JCJCJCJCHCFCFCACFCFCFCFCFCFCACDAKAFCFCFCFCFCFCFCFCFCFCICJCJCJCJC",
"JCJCJCJCJCHCFCFCFCFCFCFCFCFCFCGCFCFCFCFCFCFCFCFCFCFCHCJCJCJCJCJC",
"JCJCJCJCJCJCHCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCHCJCJCJCJCJCJC",
"JCJCJCJCJCJCJCICGCFCFCFCFCFCFCFCFCFCFCFCFCFCFCGCICJCJCJCJCJCJCJC",
"JCJCJCJCJCJCJCJCJCICGCFCFCFCFCFCFCFCFCFCFCGCHCJCJCJCJCJCJCJCJCJC",
"JCJCJCJCJCJCJCJCJCJCJCJCICHCHCHCGCHCHCJCJCJCJCJCJCJCJCJCJCJCJCJC"};


const char *UXpm::bigUp[] = {
  "32 32 42 2",
  "AA c #FFFFFFFFFFFF",
  "BA c #EFEFEFEFF7F7",
    "CA c #EFEFEFEFEFEF",
    "DA c #DEDEDEDEEFEF",
    "EA c #D6D6D6D6DEDE",
    "FA c #CECECECEE7E7",
    "GA c #BDBDBDBDDEDE",
    "HA c #B5B5B5B5BDBD",
    "IA c #ADADADADD6D6",
    "JA c #ADADADADBDBD",
    "KA c #A5A5A5A5CECE",
    "LA c #94949494ADAD",
    "MA c #8C8C8C8CBDBD",
    "NA c #8C8C8C8CADAD",
    "OA c #7B7B7B7BBDBD",
    "PA c #7B7B7B7B9C9C",
    "AB c #6B6B6B6BB5B5",
    "BB c #6B6B6B6B9C9C",
    "CB c #63636363ADAD",
    "DB c #5A5A5A5AEFEF",
    "EB c #5A5A5A5A9494",
    "FB c #4A4A4A4ADEDE",
    "GB c #4A4A4A4A9C9C",
    "HB c #4A4A4A4A7B7B",
    "IB c #42424242B5B5",
    "JB c #424242428C8C",
    "KB c #39393939CECE",
    "LB c #31313131BDBD",
    "MB c #313131319C9C",
    "NB c #313131318484",
    "OB c #29292929ADAD",
    "PB c #212121218C8C",
    "AC c #212121217B7B",
    "BC c #18181818ADAD",
    "CC c #181818189494",
    "DC c #101010105A5A",
    "EC c #080808089C9C",
    "FC c #080808088484",
    "GC c #080808086B6B",
    "HC c #000000003939",
    "IC c #000000002121",
    "JC c none",
    "JCJCJCJCJCJCJCJCJCJCJCJCJCHCHCGCHCHCHCICJCJCJCJCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCJCHCGCFCFCFCFCFCFCFCFCFCFCGCICJCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCICGCFCFCFCFCFCFCFCFCFCFCFCFCFCFCGCICJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCHCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCHCJCJCJCJCJCJC",
    "JCJCJCJCJCHCFCFCFCFCFCFCFCFCFCFCGCFCFCFCFCFCFCFCFCFCHCJCJCJCJCJC",
    "JCJCJCJCICFCFCFCFCFCFCFCFCFCFCKADAACFCFCFCFCFCFCACFCFCHCJCJCJCJC",
    "JCJCJCICFCFCFCFCFCFCFCFCFCACGAAAAACANBGCGCFCFCMADACAGAGBHCJCJCJC",
    "JCJCICFCFCFCFCFCFCFCFCFCPBDAAAAAAAAACANAFCFCKADADACAAAAAMAICJCJC",
    "JCJCFCFCECFCFCFCFCFCFCEBDAAABAAAAAAAAAAAGABBGAGADADADABABAJBJCJC",
    "JCHCECECFCECFCFCFCFCLAAAAAAAAAAAAAAAAAAAAACALAKAGAFADADACAGAICJC",
    "JCFCBCECCCECECFCECGAAAAAAAAAAAAAAABAAAAAAAAACALAKAGAFAFADADAACJC",
    "JCOBBCOBBCBCCCOBFAAAAAAAAACAAAAAAAAAAAAAAAAAAABAJALAGAFAFADACBJC",
    "HCOBOBOBOBOBBCIAAACAAAAAAAAAAAAAAAAAAACAAAAAAAAAEAMAKAGAGAFAMAIC",
    "DCKBFBLBKBLBLBKACAFAJABBBBEAAAAAAAAAAABBPALAHAEAHAPAKAKAGAGAKAHC",
    "ACFBIBDBFBFBIBLBMBACACDCFCCAAACAAAAAAAHBHBEBEBEBBBOAKAKAKAGAKAHC",
    "NBDBDBDBDBDBDBDBIBLBOBOBOBEAAAAAAAAACAEBHBEBCBABOAMAMAMAKAKAKAHC",
    "NBFBDBDBDBDBDBFBFBLBOBOBCCCAAAAAAAAAAAHBNBEBEBCBCBOAMAMAMALAKAHC",
    "ACFBFBFBFBIBFBFBLBOBOBCCCCEAAAAAAAAAAAHBACGBGBEBEBCBOAOAOAMAMAHC",
    "DCKBOBLBLBLBOBOBOBCCBCCCFCCAAAAAAAAACAHBACOBGBGBEBEBCBCBOAOACBIC",
    "ICBCOBBCOBBCBCBCECCCFCFCFCEAAAAAAAAACANBDCPBMBMBGBEBEBCBCBCBGBJC",
    "JCCCCCBCECCCCCFCFCECFCFCFCCACAAAAAAAAANBGCPBPBMBMBGBGBEBEBCBPBJC",
    "JCFCECECECECECFCFCFCFCFCFCEAAAAAAAAAAANBGCFCPBPBNBMBGBGBEBEBDCJC",
    "JCICECFCECFCFCFCFCFCFCFCGCCAAAAAAAAACANBGCFCACCCPBPBMBGBGBMBICJC",
    "JCJCFCFCFCFCFCFCFCFCFCFCFCEAAAAAAAAAAANBGCFCFCACCCPBNBNBNBGCJCJC",
    "JCJCJCFCFCFCFCFCFCFCFCFCFCCAAAAAAAAACANBGCFCFCFCFCACCCPBCCJCJCJC",
    "JCJCJCJCFCFCFCFCFCFCFCFCFCEAAAAAAAAAAANBGCFCFCFCFCFCFCFCHCJCJCJC",
    "JCJCJCJCJCFCFCFCFCFCFCFCFCJBNANANANABBGCGCFCFCFCFCFCFCICJCJCJCJC",
    "JCJCJCJCJCJCFCFCFCFCFCFCFCFCGCGCGCGCGCGCFCFCFCFCFCGCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCGCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCGCFCFCFCFCFCFCFCFCFCFCFCFCFCFCGCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCJCJCGCGCFCFCFCFCFCFCFCFCFCHCICJCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCJCJCJCJCJCICHCHCHCHCICJCJCJCJCJCJCJCJCJCJCJCJCJC",
  };
  

const char *UXpm::bigLeft[] = {
  "32 32 42 2",
  "AA c #FFFFFFFFFFFF",
  "BA c #EFEFEFEFF7F7",
  "CA c #EFEFEFEFEFEF",
  "DA c #DEDEDEDEEFEF",
  "EA c #D6D6D6D6DEDE",
  "FA c #CECECECEE7E7",
  "GA c #BDBDBDBDDEDE",
  "HA c #B5B5B5B5BDBD",
  "IA c #ADADADADD6D6",
  "JA c #ADADADADBDBD",
  "KA c #A5A5A5A5CECE",
  "LA c #94949494ADAD",
  "MA c #8C8C8C8CBDBD",
  "NA c #8C8C8C8CADAD",
  "OA c #7B7B7B7BBDBD",
  "PA c #7B7B7B7B9C9C",
  "AB c #6B6B6B6BB5B5",
  "BB c #6B6B6B6B9C9C",
  "CB c #63636363ADAD",
  "DB c #5A5A5A5AEFEF",
  "EB c #5A5A5A5A9494",
  "FB c #4A4A4A4ADEDE",
  "GB c #4A4A4A4A9C9C",
  "HB c #4A4A4A4A7B7B",
  "IB c #42424242B5B5",
  "JB c #424242428C8C",
  "KB c #39393939CECE",
  "LB c #31313131BDBD",
  "MB c #313131319C9C",
  "NB c #313131318484",
  "OB c #29292929ADAD",
  "PB c #212121218C8C",
  "AC c #212121217B7B",
  "BC c #18181818ADAD",
  "CC c #181818189494",
  "DC c #101010105A5A",
  "EC c #080808089C9C",
  "FC c #080808088484",
  "GC c #080808086B6B",
  "HC c #000000003939",
  "IC c #000000002121",
  "JC c none",
  "JCJCJCJCJCJCJCJCJCJCJCJCICHCHCHCHCHCICJCJCJCJCJCJCJCJCJCJCJCJCJC",
  "JCJCJCJCJCJCJCJCJCICACCBMAKAKAKAKAMACBGBPBDCICJCJCJCJCJCJCJCJCJC",
  "JCJCJCJCJCJCJCICJBGADADAFAGAGAKALAMAOACBCBEBMBGCJCJCJCJCJCJCJCJC",
  "JCJCJCJCJCJCHCMABACADAFAGAGAKAKAMAOAOACBEBEBGBNBCCHCJCJCJCJCJCJC",
  "JCJCJCJCJCHCGBAABADAFAFAGAKAKAMAMAOACBCBEBGBGBNBPBFCICJCJCJCJCJC",
  "JCJCJCJCHCFCGAAADADAFAGAKAKAKAMAMAOACBEBGBGBMBNBCCFCFCJCJCJCJCJC",
  "JCJCJCHCFCFCCACADAFAGALAMAPAOAMAOACBEBEBGBMBPBPBACFCFCGCJCJCJCJC",
  "JCJCICFCFCACDADADAGAKAJAEAHABBOACBEBEBGBMBNBPBCCFCFCFCFCGCJCJCJC",
  "JCJCGCFCFCFCMADAGAKALABAAAEAEBABCBEBGBMBMBPBCCACFCFCFCFCFCGCJCJC",
  "JCICFCFCFCFCFCKAGALACAAAAAHAEBCBEBGBGBMBPBPBACFCFCFCFCFCFCFCICJC",
  "JCGCFCFCFCFCFCFCBBCAAAAAAALAEBEBEBGBOBPBPBFCFCFCFCFCFCFCFCFCHCJC",
  "JCFCFCFCFCFCGCFCGAAAAAAAAAPAHBHBNBACACDCGCGCGCGCGCGCGCFCFCFCFCJC",
  "ICFCFCFCFCFCGCNAAAAAAAAACABBHBEBHBHBHBNBNBNBNBNBNBNBGCGCFCFCFCJC",
  "HCFCFCFCFCFCNBCAAAAAAAAAAAAAAACAAAAACACAAAAACAAACAAABBGCFCFCFCIC",
  "HCFCFCFCFCACCAAAAAAABAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAANAGCFCFCFCHC",
  "HCFCFCFCGCDAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAANAGCFCFCFCHC",
  "GCFCFCFCFCKAAAAAAAAAAAAAAAAACAAAAAAAAAAAAAAAAAAAAAAANAGCFCFCFCHC",
  "HCFCFCFCFCFCGAAABAAAAAAAAAAAAAAAAAAAAAAACAAAAAAAAAAANAGCFCFCFCHC",
  "HCFCFCFCFCFCACDAAAAAAACAAAEACAEACAEACAEACAEACAEACAEAJBFCFCFCFCIC",
  "JCFCFCFCFCFCFCPBDAAAAAAAAABBFCOBCCCCFCFCFCFCGCFCFCFCFCFCFCFCFCJC",
  "JCFCFCFCFCFCFCFCEBAAAAAAAABBDCOBOBCCCCFCFCFCFCFCFCFCFCFCFCFCGCJC",
  "JCGCFCFCFCFCFCFCFCLAAAAAAAJAACOBOBOBBCFCFCFCFCFCFCFCFCFCFCFCGCJC",
  "JCHCFCFCFCFCFCFCFCFCGAAACAFAACLBLBOBCCCCECFCFCFCFCFCFCFCFCFCJCJC",
  "JCJCGCFCFCFCFCFCFCFCECFAAACAMBIBFBLBOBECFCFCFCFCFCFCFCFCFCGCJCJC",
  "JCJCICFCFCFCFCFCFCFCFCOBIAKALBDBFBFBOBBCFCFCFCFCFCFCFCFCFCJCJCJC",
  "JCJCJCHCFCFCFCFCFCFCECCCBCLBIBDBDBFBOBBCCCECFCFCFCFCFCFCJCJCJCJC",
  "JCJCJCJCHCFCFCFCFCECECBCOBLBFBDBDBIBLBBCCCECFCFCFCFCFCJCJCJCJCJC",
  "JCJCJCJCJCICFCFCECFCCCBCOBKBFBDBDBFBLBOBECECECFCFCFCJCJCJCJCJCJC",
  "JCJCJCJCJCJCICFCFCECECOBOBLBDBDBDBFBLBBCBCECFCFCFCJCJCJCJCJCJCJC",
  "JCJCJCJCJCJCJCICFCECBCBCOBFBIBDBDBFBOBOBCCECECFCJCJCJCJCJCJCJCJC",
  "JCJCJCJCJCJCJCJCJCHCFCOBOBKBFBDBFBFBKBBCCCFCICJCJCJCJCJCJCJCJCJC",
  "JCJCJCJCJCJCJCJCJCJCJCJCHCDCACNBNBACDCICJCJCJCJCJCJCJCJCJCJCJCJC",
};

const char *UXpm::bigRight[] = {
    "32 32 42 2",
    "AA c #FFFFFFFFFFFF",
    "BA c #EFEFEFEFF7F7",
    "CA c #EFEFEFEFEFEF",
    "DA c #DEDEDEDEEFEF",
    "EA c #D6D6D6D6DEDE",
    "FA c #CECECECEE7E7",
    "GA c #BDBDBDBDDEDE",
    "HA c #B5B5B5B5BDBD",
    "IA c #ADADADADD6D6",
    "JA c #ADADADADBDBD",
    "KA c #A5A5A5A5CECE",
    "LA c #94949494ADAD",
    "MA c #8C8C8C8CBDBD",
    "NA c #8C8C8C8CADAD",
    "OA c #7B7B7B7BBDBD",
    "PA c #7B7B7B7B9C9C",
    "AB c #6B6B6B6BB5B5",
    "BB c #6B6B6B6B9C9C",
    "CB c #63636363ADAD",
    "DB c #5A5A5A5AEFEF",
    "EB c #5A5A5A5A9494",
    "FB c #4A4A4A4ADEDE",
    "GB c #4A4A4A4A9C9C",
    "HB c #4A4A4A4A7B7B",
    "IB c #42424242B5B5",
    "JB c #424242428C8C",
    "KB c #39393939CECE",
    "LB c #31313131BDBD",
    "MB c #313131319C9C",
    "NB c #313131318484",
    "OB c #29292929ADAD",
    "PB c #212121218C8C",
    "AC c #212121217B7B",
    "BC c #18181818ADAD",
    "CC c #181818189494",
    "DC c #101010105A5A",
    "EC c #080808089C9C",
    "FC c #080808088484",
    "GC c #080808086B6B",
    "HC c #000000003939",
    "IC c #000000002121",
    "JC c none",
    "JCJCJCJCJCJCJCJCJCJCJCJCICDCACNBNBACDCHCJCJCJCJCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCJCICFCCCBCKBFBFBDBFBKBOBOBFCHCJCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCFCECECCCOBOBFBDBDBIBFBOBBCBCECFCICJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCFCFCFCECBCBCLBFBDBDBDBLBOBOBECECFCFCICJCJCJCJCJCJC",
    "JCJCJCJCJCJCFCFCFCECECECOBLBFBDBDBFBKBOBBCCCFCECFCFCICJCJCJCJCJC",
    "JCJCJCJCJCFCFCFCFCFCECCCBCLBIBDBDBFBLBOBBCECECFCFCFCFCHCJCJCJCJC",
    "JCJCJCJCFCFCFCFCFCFCECCCBCOBFBDBDBIBLBBCCCECFCFCFCFCFCFCHCJCJCJC",
    "JCJCJCFCFCFCFCFCFCFCFCFCBCOBFBFBDBLBKAIAOBFCFCFCFCFCFCFCFCICJCJC",
    "JCJCGCFCFCFCFCFCFCFCFCFCECOBLBFBIBMBCAAAFAECFCFCFCFCFCFCFCGCJCJC",
    "JCJCFCFCFCFCFCFCFCFCFCECCCCCOBLBLBACFACAAAGAFCFCFCFCFCFCFCFCHCJC",
    "JCGCFCFCFCFCFCFCFCFCFCFCFCBCOBOBOBACJAAAAAAALAFCFCFCFCFCFCFCGCJC",
    "JCGCFCFCFCFCFCFCFCFCFCFCFCCCCCOBOBDCBBAAAAAAAAEBFCFCFCFCFCFCFCJC",
    "JCFCFCFCFCFCFCFCFCGCFCFCFCFCCCCCOBFCBBAAAAAAAADAPBFCFCFCFCFCFCJC",
    "ICFCFCFCFCJBEACAEACAEACAEACAEACAEACAEAAACAAAAAAADAACFCFCFCFCFCHC",
    "HCFCFCFCGCNAAAAAAAAAAACAAAAAAAAAAAAAAAAAAAAAAABAAAGAFCFCFCFCFCHC",
    "HCFCFCFCGCNAAAAAAAAAAAAAAAAAAAAAAACAAAAAAAAAAAAAAAAAKAFCFCFCFCGC",
    "HCFCFCFCGCNAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAADAGCFCFCFCHC",
    "HCFCFCFCGCNAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABAAAAAAACAACFCFCFCFCHC",
    "ICFCFCFCGCBBAACAAACAAAAACACAAAAACAAAAAAAAAAAAAAACANBFCFCFCFCFCHC",
    "JCFCFCFCGCGCNBNBNBNBNBNBNBHBHBHBEBHBBBCAAAAAAAAANAGCFCFCFCFCFCIC",
    "JCFCFCFCFCGCGCGCGCGCGCGCDCACACNBHBHBPAAAAAAAAAGAFCGCFCFCFCFCFCJC",
    "JCHCFCFCFCFCFCFCFCFCFCPBPBOBGBEBEBEBLAAAAAAACABBFCFCFCFCFCFCGCJC",
    "JCICFCFCFCFCFCFCFCACPBPBMBGBGBEBCBEBHAAAAACALAGAKAFCFCFCFCFCICJC",
    "JCJCGCFCFCFCFCFCACCCPBMBMBGBEBCBABEBEAAABALAKAGADAMAFCFCFCGCJCJC",
    "JCJCJCGCFCFCFCFCCCPBNBMBGBEBEBCBOABBHAEAJAKAGADADADAACFCFCICJCJC",
    "JCJCJCJCGCFCFCACPBPBMBGBEBEBCBOAMAOAPAMALAGAFADACACAFCFCHCJCJCJC",
    "JCJCJCJCJCFCFCCCNBMBGBGBEBCBOAMAMAKAKAKAGAFADADAAAGAFCHCJCJCJCJC",
    "JCJCJCJCJCICFCPBNBGBGBEBCBCBOAMAMAKAKAGAFAFADABAAAGBHCJCJCJCJCJC",
    "JCJCJCJCJCJCHCCCNBGBEBEBCBOAOAMAKAKAGAGAFADACABAMAHCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCGCMBEBCBCBOAMALAKAGAGAFADADAGAJBICJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCJCICDCPBGBCBMAKAKAKAKAMACBACICJCJCJCJCJCJCJCJCJC",
    "JCJCJCJCJCJCJCJCJCJCJCJCJCICHCHCHCHCHCICJCJCJCJCJCJCJCJCJCJCJCJC",
  };
  
//============================================================================

const char * UXpm::bigPlus[] = {
  "25 25 11 1",
  "A c #E7E7F7F7F7F7",
  "B c #DEDEEFEFF7F7",
  "C c #8484B5B5BDBD",
  "D c #7B7BADADBDBD",
  "E c #393984849C9C",
    "F c #31317B7B8C8C",
  "G c #10105A5A7373",
  "H c #08084A4A5A5A",
  "I c #000029293939",
  "J c #000010101818",
  "K c None s None",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKJJKKKKKKKKKKKK",
  "KKKKKKKKKKIJIIJJKKKKKKKKK",
  "KKKKKKKKKJJCCDIKKKKKKKKKK",
  "KKKKKKKKKJIBFAIJJKKKKKKKK",
  "KKKKKKKKJIIAGBIJKJKKKKKKK",
  "KKKKKKJJJJHBGBHIIJJKKKKKK",
  "KKKKKJIIHHHAGBHHJIIJKKKKK",
  "KKKKJKFCCCCBGACCCCCIJKKKK",
  "KKKKJICDFEEEGEEEEEAJJKKKK",
  "KKKKJKDBBAAAGAAAAAAIJJKKK",
  "KKKKKIKIIHGAGAHHIIIKKKKKK",
  "KKKKKKJJJIHAGBGIIJKJJKKKK",
  "KKKKKKKKJJHBHAIJKJKKKKKKK",
  "KKKKKKKKKJIAHBHIKKKKKKKKK",
  "KKKKKKKKKJIAAAKKJKKKKKKKK",
  "KKKKKKKKKKJJIJIJKKKKKKKKK",
  "KKKKKKKKKJKJJJKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
  "KKKKKKKKKKKKKKKKKKKKKKKKK",
};

const char *UXpm::bigMinus[] = {
  "19 15 11 1",
    "A c #FFFFFFFFFFFF",
    "B c #E7E7F7F7F7F7",
    "C c #8484ADADB5B5",
    "D c #393984849C9C",
    "E c #393973738484",
    "F c #080831313939",
    "G c #000029293131",
    "H c #000018182121",
    "I c #000018181818",
    "J c #000008081010",
    "K c None s None",
    "KKKKKKKKKKKKKKKKKKK",
    "KKKKKKKKKKKKKKKKKKK",
    "KKKKKKKKKKKKKKKKKKK",
    "KKKKKKKKKKKKKKKKKKK",
    "KKKKKKIIIIIIKKKKKKK",
    "KKKKJHHGGGGHGHKKKKK",
    "KKKKIGCCCCCCEHIKKKK",
    "KKKKJGADDDDCCGJKKKK",
    "KKKKIGABABBBCHIKKKK",
    "KKKKJJGFFFFGGIKKKKK",
    "KKKKKIJHHGHHJKJKKKK",
    "KKKKKKKKJKJKKKKKKKK",
    "KKKKKKKKKKKKKKKKKKK",
    "KKKKKKKKKKKKKKKKKKK",
    "KKKKKKKKKKKKKKKKKKK",
};

const char *UXpm::bigQuestion[] = {
  "24 31 18 1",
  "A c #FFFFFFFFFFFF",
  "B c #DEDEEFEFF7F7",
  "C c #C6C6DEDEE7E7",
  "D c #B5B5D6D6DEDE",
  "E c #9494C6C6CECE",
  "F c #7B7BB5B5C6C6",
  "G c #52529C9CADAD",
  "H c #42429C9CADAD",
  "I c #313184849C9C",
  "J c #212173738484",
  "K c #212163637373",
  "L c #18186B6B7B7B",
  "M c #18185A5A6B6B",
  "N c #080852526363",
  "O c #080839394A4A",
  "P c #000029293131",
  "Q c #000010101818",
  "R c None s None",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRQQQQQRRRRRRRRR",
  "RRRRRRRRQQPPPOQPQRRRRRRR",
  "RRRRRRRQPOKEFFKOPQRRRRRR",
  "RRRRRRQOOEBEIFACOPQRRRRR",
  "RRRRRQPOBCIIIIJCCOPRRRRR",
  "RRRRQRPGCHAEFDAICFPPRRRR",
  "RRRRRQOAGEFMOMEEGCPRQRRR",
  "RRRRRQPCBAMPOOJFKBOPRRRR",
  "RRRRRRPPOPQPQMIEIAPQRRRR",
  "RRRRRRRQRQQQOOCEDEOQRRRR",
  "RRRRRRRRQROQKCAEBMQQRRRR",
  "RRRRRRRRRQRMCCEAJPPRRRRR",
  "RRRRRRRRRQOGBFBJOQRRRRRR",
  "RRRRRRRRQQOAGCHOPQRRRRRR",
  "RRRRRRRRROKBLAOORRRRRRRR",
  "RRRRRRRRQRKDECMRQRRRRRRR",
  "RRRRRRRRROOJLJQORRRRRRRR",
  "RRRRRRRRQROKKJORRRRRRRRR",
  "RRRRRRRRROKCFBORQRRRRRRR",
  "RRRRRRRRQRKFNAQORRRRRRRR",
  "RRRRRRRRROKABCORRRRRRRRR",
  "RRRRRRRRRRQQQORQRRRRRRRR",
  "RRRRRRRRRRQQQRQRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
  "RRRRRRRRRRRRRRRRRRRRRRRR",
};

/* ==================================================== ======== ======= */
/*
 const char* UXpm::left[] = {
 "17 17 5 1",
 "A c None s None",
 "B c #DEDEDEDEDEDE",
 "C c #B5B5B5B5B5B5",
 "D c #8C8C8C8C8C8C",
 "E c #6B6B6B6B6B6B",
 "AAAAAAAAAAAAAAAAA",
 "AAAAAAAAAAAAAAAAA",
 "AAAAACDDEDDCAAAAA",
 "AAABCEEEEEEECBAAA",
 "AAACEEEEEEEEEDAAA",
 "AACEEEEEEEEEEECAA",
 "AADEEEEECAEEEEDAA",
 "AAEEEECAAAEEEEEAA",
 "AAEEECAAAAEEEEEAA",
 "AADEEECAAAEEEEEAA",
 "AADEEEEECAEEEEDAA",
 "AACEEEEEEEEEEECAA",
 "AAADEEEEEEEEEDBAA",
 "AAAADEEEEEEEDBAAA",
 "AAAAACDEEEDCAAAAA",
 "AAAAAAAAAAAAAAAAA",
 "AAAAAAAAAAAAAAAAA",
 };
 
 const char *UXpm::right[] = {
 "17 17 5 1",
 "A c None s None",
 "B c #DEDEDEDEDEDE",
 "C c #B5B5B5B5B5B5",
 "D c #8C8C8C8C8C8C",
 "E c #6B6B6B6B6B6B",
 "AAAAAAAAAAAAAAAAA",
 "AAAAAAAAAAAAAAAAA",
 "AAAAACDEEEDCAAAAA",
 "AAABDEEEEEEEDAAAA",
 "AABDEEEEEEEEEDAAA",
 "AACEEEEEEEEEEECAA",
 "AADEEEEACEEEEEDAA",
 "AAEEEEEAAACEEEDAA",
 "AAEEEEEAAAACEEEAA",
 "AAEEEEEAAACEEEEAA",
 "AADEEEEACEEEEEDAA",
 "AACEEEEEEEEEEECAA",
 "AAADEEEEEEEEECAAA",
 "AAABCEEEEEEECBAAA",
 "AAAAACDDEDDCAAAAA",
 "AAAAAAAAAAAAAAAAA",
 "AAAAAAAAAAAAAAAAA",
 };
 
 const char* UXpm::up[] = {
 "17 17 5 1",
 "A c None s None",
 "B c #DEDEDEDEDEDE",
 "C c #B5B5B5B5B5B5",
 "D c #8C8C8C8C8C8C",
 "E c #6B6B6B6B6B6B",
 "AAAAAAAAAAAAAAAAA",
 "AAAAAAAAAAAAAAAAA",
 "AAAAACDDEEDCAAAAA",
 "AAAADEEEEEEECBAAA",
 "AAADEEEEEEEEECAAA",
 "AACEEEEECEEEEECAA",
 "AADEEEECACEEEEDAA",
 "AAEEEEEAAAEEEEDAA",
 "AAEEEECAAACEEEEAA",
 "AAEEEEAAAAAEEEDAA",
 "AADEEEEEEEEEEEDAA",
 "AACEEEEEEEEEEECAA",
 "AAADEEEEEEEEECAAA",
 "AAABDEEEEEEEDBAAA",
 "AAAABCDEEEDCAAAAA",
 "AAAAAAAAAAAAAAAAA",
 "AAAAAAAAAAAAAAAAA",
 };
 
 
 const char* UXpm::down[] ={
 "17 17 5 1",
 "A c #DEDEDEDEDEDE",
 "B c #B5B5B5B5B5B5",
 "C c #8C8C8C8C8C8C",
 "D c #6B6B6B6B6B6B",
 "E c None",
 "EEEEEEEEEEEEEEEEE",
 "EEEEEEEEEEEEEEEEE",
 "EEEEEBCDDDCBAEEEE",
 "EEEACDDDDDDDCAEEE",
 "EEEBDDDDDDDDDCEEE",
 "EEBDDDDDDDDDDDBEE",
 "EECDDDDDDDDDDDCEE",
 "EECDDDEEEEEDDDDEE",
 "EEDDDDBEEEBDDDDEE",
 "EECDDDDEEEDDDDDEE",
 "EECDDDDBEBDDDDCEE",
 "EEBDDDDDBDDDDDBEE",
 "EEEBDDDDDDDDDCEEE",
 "EEEABDDDDDDDCEEEE",
 "EEEEEBCDDCCBEEEEE",
 "EEEEEEEEEEEEEEEEE",
 "EEEEEEEEEEEEEEEEE",
 };
 */
/* ==================================================== ======== ======= */

UPix UPix::bigFolder(UXpm::bigFolder,  UObject::UCONST);
UPix UPix::bigDoc(UXpm::bigDoc,  UObject::UCONST);
UPix UPix::bigText(UXpm::bigText,  UObject::UCONST);
UPix UPix::bigIma(UXpm::bigIma,  UObject::UCONST);
UPix UPix::bigUp(UXpm::bigUp,  UObject::UCONST);
UPix UPix::bigDown(UXpm::bigDown,  UObject::UCONST);
UPix UPix::bigLeft(UXpm::bigLeft,  UObject::UCONST);
UPix UPix::bigRight(UXpm::bigRight,  UObject::UCONST);

//UPix UPix::edit(UXpm::edit,  UObject::UCONST);
//UPix UPix::colors(UXpm::colors,  UObject::UCONST);
//UPix UPix::windows(UXpm::windows,  UObject::UCONST);
//UPix UPix::clock(UXpm::clock, UMode::UCONST);
//UPix UPix::bomb(UXpm::bomb, UMode::UCONST);
//UPix UPix::book(UXpm::book, UObject::UCONST);  
//UPix UPix::eyes(UXpm::eyes,  UObject::UCONST);
//UPix UPix::leftarrow(UXpm::leftarrow,  UObject::UCONST);
//UPix UPix::rightarrow(UXpm::rightarrow,  UObject::UCONST);
//UPix UPix::uparrow(UXpm::uparrow,  UObject::UCONST);
//UPix UPix::downarrow(UXpm::downarrow,  UObject::UCONST);
/*
 UPix UPix::ellipsis(UXpm::ellipsis,  UCONST);
 UPix UPix::cross(UXpm::cross,  UCONST);
 UPix UPix::check(UXpm::check,  UCONST);
 UPix UPix::select(UXpm::select,  UCONST);
 UPix UPix::left(UXpm::left,  UCONST);
 UPix UPix::right(UXpm::right,  UCONST);
 UPix UPix::up(UXpm::up,  UCONST);
 UPix UPix::down(UXpm::down,  UCONST);
 UPix UPix::rball(UXpm::rball,  UCONST);
 */
}
