/* ***********************************************************************
 *
 *  CONFIGURE.cpp: Project Configuration for creating Makefiles
 *
 *  *** NOTE: if the ./umake command (located in the Ubit base directory)  
 *  *** does not not work properly, you'll have to edit this file to set LIBRARY 
 *  *** PATHS and COMPILATION OPTIONS appropriately
 *
 *  ALTERNATIVELY, you can also launch ./configure in the Ubit base directory
 *
 *  Ubit GUI Toolkit - Version 6.3
 *  (C) 2009 Eric Lecolinet | ENST Paris | www.enst.fr/~elc/ubit
 *
 * ***********************************************************************
 * COPYRIGHT NOTICE : 
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY AND WITHOUT EVEN THE 
 * IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER THE TERMS OF THE GNU 
 * GENERAL PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
 * EITHER VERSION 2 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
 * SEE FILES 'COPYRIGHT' AND 'COPYING' FOR MORE DETAILS.
 * ***********************************************************************/


// see comments in umake.hpp for explanations on the variables used below.
#include "umake.hpp"
using namespace std;

void UMake::setProjectOptions()
{  
  PROJECT_NAME = "UBIT";
  PROJECT_VERSION = "6.3.0";
  
   // must be one of X11 or GLUT
  WIN_TOOLKIT = getVar("WIN_TOOLKIT", "");
  
  if (WIN_TOOLKIT != "GLUT" && WIN_TOOLKIT != "X11") {
    cout << "umake: wrong configuration parameter: WIN_TOOLKIT must be one of GLUT or X11" << endl;
    exit(1);
  }
    
  // Compilers and Compiler flags
  CC       = getVar("CC", "gcc");
  CXX      = getVar("CXX", "g++");
  CFLAGS   = getVar("CFLAGS", "");
  CXXFLAGS = getVar("CXXFLAGS", "");
  
  // Useful commands
  RM       = getVar("RM", "rm");
  CP       = getVar("CP", "cp");
  INSTALL  = getVar("INSTALL", "install");
  
   
  // Build directories
  string BUILD_DIR = getVar("BUILD_DIR","");
  BUILD_INCLUDE_DIR = BUILD_DIR + "/src";
  BUILD_LIB_DIR = BUILD_DIR + "/src/ubit";
  BUILD_BIN_DIR = BUILD_DIR + "/bin";

  // Project library(ies)
  PROJECT_LIBS = "-lubit";

  // Generated files
  FEATURES_HEADER = BUILD_DIR + "/src/ubit/ubit_features.h";  
  MAKEFILE_DEFS = "Makefile-defs";

  // Installation directories
  string INSTALL_DIR = getVar("INSTALL_DIR","");
  INSTALL_INCLUDE_DIR = INSTALL_DIR + "/include";
  INSTALL_LIB_DIR = INSTALL_DIR + "/lib";
  INSTALL_BIN_DIR = INSTALL_DIR + "/bin";
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void UMake::setProjectConfig()
{
  // *** FIRST STEP: SETTING PATHS ***
  //
  // For each library, specify were its headers and binaries can be found according
  // to the OS (which is specified by 'SYSNAME')
  // (the 1st arg of add() is the header path, the second arg is the bin path)
  
  Paths glut_paths, x11_paths, freetype_paths, ima_paths;

  if (SYSNAME == "Darwin") {
    x11_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    //freetype_paths.add("-I/usr/X11R6/include -I/usr/X11R6/include/freetype2", "-L/usr/X11R6/lib");
    freetype_paths.add("-I/opt/local/include/freetype2", "-L/opt/local/lib");
    ima_paths.add("-I/opt/local/include", "-L/opt/local/lib");  // MacPorts
    //ima_paths.add("-I/sw/include", "-L/sw/lib"); // Fink
  }
  
  else if (SYSNAME == "SunOS") {
    glut_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    glut_paths.add("-I/usr/local/include", "-L/usr/local/lib");
    x11_paths.add("-I/usr/openwin/include", "-L/usr/openwin/lib");
    x11_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    freetype_paths.add("-I/usr/X11R6/include -I/usr/X11R6/include/freetype2", "-L/usr/X11R6/lib");
    freetype_paths.add("-I/usr/local/include -I/usr/local/include/freetype2", "-L/usr/local/lib");
    ima_paths.add("-I/usr/local/include", "-L/usr/local/lib");
  }
  
  else if (SYSNAME == "Linux") {
    glut_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    glut_paths.add("-I/usr/include", "-L/usr/lib");
    x11_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    x11_paths.add("-I/usr/include", "-L/usr/lib");
    freetype_paths.add("-I/usr/X11R6/include -I/usr/X11R6/include/freetype2", "-L/usr/X11R6/lib");
    freetype_paths.add("-I/usr/include -I/usr/include/freetype2", "-L/usr/lib");
    ima_paths.add("-I/usr/include", "-L/usr/lib");
  }
  
  else if (SYSNAME == "Netbsd") {
    glut_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    glut_paths.add("-I/usr/pkg/include", "-L/usr/pkg/lib");
    x11_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    x11_paths.add("-I/usr/pkg/include", "-L/usr/pkg/lib");
    freetype_paths.add("-I/usr/X11R6/include -I/usr/X11R6/include/freetype2", "-L/usr/X11R6/lib");
    freetype_paths.add("-I/usr/pkg/include -I/usr/pkg/include/freetype2", "-L/usr/pkg/lib");
    ima_paths.add("-I/usr/pkg/include", "-L/usr/pkg/lib");
  }
  
  else {
    cout << "! Unknown operating System: '" << SYSNAME << "' ; using default directives"<< endl;
    glut_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    glut_paths.add("-I/usr/local/include", "-L/usr/local/lib");
    x11_paths.add("-I/usr/X11R6/include", "-L/usr/X11R6/lib");
    x11_paths.add("-I/usr/local/include", "-L/usr/local/lib");
    freetype_paths.add("-I/usr/X11R6/include/freetype2", "-L/usr/X11R6/lib");
    freetype_paths.add("-I/usr/local/include/freetype2", "-L/usr/local/lib");
    ima_paths.add("-I/usr/local/include", "-L/usr/local/lib");
  }

  
  // *** SECOND STEP: SEARCHING FOR LIBRARIES ***
  //
  // This will check if the library headers and binaries can really be found in
  // order to create the configuration files. Note that order matters.
  // - needLib() specifies a lib that is mandatory to build the project
  // - optLib()  specifies an optional lib (the project can be build if it does no exist)
  //
  // Syntax: needLib(name, paths, headers, binaries, function)  (same syntax for optLib())
  // with;
  // - name: the lib name. The configuration file 'ubit_features.h' will contain
  //         #define UBIT_WITH_name 1
  //   if this lib (and its headers) is found
  // - paths: the paths object specified for this lib in STEP 1
  // - headers: a header file that is required to use this lib (*)
  // - binaries: the required binarie(s), given by -l or -framework specifications
  //   or absolute paths of the appopriate .a, .so, .dylib... binary files 
  // - function: a C function of the lib
  //
  // The existence of the lib is checked by compiling a program that calls the
  // 'function' using the specified 'headers' and 'binaries'
  //
  // (*) Note: several headers can be specified in which case the complete
  // expression must be given with #include statements such as follows:
  //         "#include<ft2build.h>\n#include FT_FREETYPE_H" 

  
  // using GLUT and GL - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  
  if (WIN_TOOLKIT == "GLUT") {
    if (SYSNAME == "Darwin") {
      needLib("GLUT", glut_paths, "GLUT/glut.h", "-framework GLUT", "glutMainLoop");
      
      // works from MACOSX 10.5.7
      needLib("GL", glut_paths, "OpenGL/gl.h", "-framework OpenGL", "glGetError");
      
      // was needed before MACOSX 10.5.7 beacuse of a bug
      //needLib("GL", x11_paths, "GL/gl.h", "-lGLU -lGL /System/Library/Frameworks/OpenGL.framework/Libraries/libGLU.dylib /System/Library/Frameworks/OpenGL.framework/Libraries/libGL.dylib", "glGetError");
    }
    else {
      needLib("GLUT", glut_paths, "GL/glut.h", "-lglut", "glutMainLoop");
      needLib("GL", x11_paths, "GL/gl.h", "-lGLU -lGL", "glGetError");
    }
    
    needLib("FREETYPE", freetype_paths, "#include<ft2build.h>\n#include FT_FREETYPE_H", "-lfreetype", "FT_Init_FreeType");
  }
  
  // using X11 and GL - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  else if (WIN_TOOLKIT == "X11") {
    if (SYSNAME == "Darwin") {
      // needLib("GL", glut_paths, "OpenGL/gl.h", "-framework OpenGL", "glGetError");
      // this is because of a bug of on Leopard when using X11 + GL
      needLib("GL", x11_paths, "GL/gl.h", "-lGLU -lGL /System/Library/Frameworks/OpenGL.framework/Libraries/libGLU.dylib /System/Library/Frameworks/OpenGL.framework/Libraries/libGL.dylib", "glGetError");
    } 
    else {
      needLib("GL", x11_paths, "GL/gl.h", "-lGLU -lGL", "glGetError");
    }
    needLib("X11", x11_paths, "X11/Xlib.h", "-lXmu -lXtst -lXext -lX11", "XOpenDisplay");
    optLib("XPM",  x11_paths, "X11/xpm.h", "-lXpm -lX11", "XpmGetErrorString");
    needLib("FREETYPE", freetype_paths, "#include<ft2build.h>\n#include FT_FREETYPE_H", "-lfreetype", "FT_Init_FreeType");
  }
  
  // using X11 without GL (obsolete) - - - - - - - - - - - - - - - - - - - - - -
  // else if (WIN_TOOLKIT == "X11nogl") {
  //   needLib("X11", x11_paths, "X11/Xlib.h", "-lXmu -lXtst -lXext -lX11", "XOpenDisplay");
  //  optLib("XPM",  x11_paths, "X11/xpm.h", "-lXpm -lX11", "XpmGetErrorString");
  //}
  
  // in all cases - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  // GIF or UNGIF
  if (!optLib("GIF", ima_paths, "gif_lib.h", "-lgif", "DGifOpenFileName")) {
    remLib("GIF");
    optLib("UNGIF", ima_paths, "gif_lib.h", "-lungif", "DGifOpenFileName");
  }
  
  // JPEG
  optLib("JPEG", ima_paths, "jpeglib.h", "-ljpeg", "jpeg_read_scanlines");
}


